"""Tests for SovereigntyAssessor and SovereigntyAssessment."""
from __future__ import annotations

import pytest

from agent_sovereign.classifier.assessor import (
    SovereigntyAssessment,
    SovereigntyAssessor,
    _DEPLOYMENT_TEMPLATE_NAMES,
    _GEOGRAPHY_MINIMUMS,
)
from agent_sovereign.classifier.levels import SovereigntyLevel


# ---------------------------------------------------------------------------
# Fixtures
# ---------------------------------------------------------------------------

@pytest.fixture()
def assessor() -> SovereigntyAssessor:
    return SovereigntyAssessor()


# ---------------------------------------------------------------------------
# SovereigntyAssessment dataclass
# ---------------------------------------------------------------------------

class TestSovereigntyAssessment:
    def test_fields_accessible(self) -> None:
        assessment = SovereigntyAssessment(
            level=SovereigntyLevel.L3_HYBRID,
            score=3,
            justification="test",
            data_sensitivity=2,
            regulatory_drivers={"HIPAA": SovereigntyLevel.L3_HYBRID},
            deployment_template="l3_hybrid",
        )
        assert assessment.score == 3
        assert assessment.level == SovereigntyLevel.L3_HYBRID
        assert assessment.deployment_template == "l3_hybrid"
        assert assessment.warnings == []
        assert assessment.capability_requirements == {}

    def test_default_warnings_empty(self) -> None:
        assessment = SovereigntyAssessment(
            level=SovereigntyLevel.L1_CLOUD,
            score=1,
            justification="baseline",
            data_sensitivity=1,
            regulatory_drivers={},
            deployment_template="l1_cloud",
        )
        assert assessment.warnings == []

    def test_warnings_can_be_set(self) -> None:
        warnings_list = ["warning one"]
        assessment = SovereigntyAssessment(
            level=SovereigntyLevel.L2_CLOUD_DEDICATED,
            score=2,
            justification="test",
            data_sensitivity=1,
            regulatory_drivers={},
            deployment_template="l2_cloud_fallback",
            warnings=warnings_list,
        )
        assert len(assessment.warnings) == 1
        assert assessment.warnings[0] == "warning one"


# ---------------------------------------------------------------------------
# Geography minimums constant
# ---------------------------------------------------------------------------

class TestGeographyMinimums:
    def test_eu_is_level_6(self) -> None:
        assert _GEOGRAPHY_MINIMUMS["EU"] == 6

    def test_us_is_level_2(self) -> None:
        assert _GEOGRAPHY_MINIMUMS["US"] == 2

    def test_global_is_level_1(self) -> None:
        assert _GEOGRAPHY_MINIMUMS["GLOBAL"] == 1

    def test_us_dod_is_level_5(self) -> None:
        assert _GEOGRAPHY_MINIMUMS["US_DOD"] == 5


# ---------------------------------------------------------------------------
# Deployment template names
# ---------------------------------------------------------------------------

class TestDeploymentTemplateNames:
    def test_all_levels_covered(self) -> None:
        for level_value in range(1, 8):
            assert level_value in _DEPLOYMENT_TEMPLATE_NAMES

    def test_level_7_is_airgapped(self) -> None:
        assert _DEPLOYMENT_TEMPLATE_NAMES[7] == "l7_airgapped"

    def test_level_1_is_cloud(self) -> None:
        assert _DEPLOYMENT_TEMPLATE_NAMES[1] == "l1_cloud"


# ---------------------------------------------------------------------------
# SovereigntyAssessor construction
# ---------------------------------------------------------------------------

class TestAssessorConstruction:
    def test_default_org_minimum_is_l1(self) -> None:
        assessor = SovereigntyAssessor()
        assert assessor._org_minimum == SovereigntyLevel.L1_CLOUD

    def test_custom_org_minimum(self) -> None:
        assessor = SovereigntyAssessor(org_minimum=SovereigntyLevel.L3_HYBRID)
        assert assessor._org_minimum == SovereigntyLevel.L3_HYBRID

    def test_additional_geo_minimums_merged(self) -> None:
        assessor = SovereigntyAssessor(additional_geo_minimums={"XX": 5})
        assert assessor._geo_minimums["XX"] == 5
        assert "EU" in assessor._geo_minimums  # built-ins preserved

    def test_additional_geo_minimums_override(self) -> None:
        assessor = SovereigntyAssessor(additional_geo_minimums={"EU": 7})
        assert assessor._geo_minimums["EU"] == 7


# ---------------------------------------------------------------------------
# SovereigntyAssessor.assess — baseline
# ---------------------------------------------------------------------------

class TestAssessorAssess:
    def test_baseline_no_inputs_returns_l1(self, assessor: SovereigntyAssessor) -> None:
        result = assessor.assess(data_types=[], regulations=[])
        assert result.score == 1
        assert result.level == SovereigntyLevel.L1_CLOUD
        assert result.deployment_template == "l1_cloud"

    def test_phi_data_drives_level_up(self, assessor: SovereigntyAssessor) -> None:
        result = assessor.assess(data_types=["phi"], regulations=[])
        assert result.score >= 3

    def test_hipaa_regulation_drives_level(self, assessor: SovereigntyAssessor) -> None:
        result = assessor.assess(data_types=[], regulations=["HIPAA"])
        assert result.score >= 3

    def test_eu_geography_drives_level_6(self, assessor: SovereigntyAssessor) -> None:
        result = assessor.assess(data_types=[], regulations=[], geography="EU")
        assert result.score >= 6

    def test_unknown_geography_produces_warning(self, assessor: SovereigntyAssessor) -> None:
        result = assessor.assess(data_types=[], regulations=[], geography="ZZZZ")
        assert any("ZZZZ" in w for w in result.warnings)

    def test_unknown_regulation_produces_warning(self, assessor: SovereigntyAssessor) -> None:
        result = assessor.assess(data_types=[], regulations=["UNKNOWN_REG"])
        assert any("UNKNOWN_REG" in w for w in result.warnings)

    def test_org_minimum_overrides_per_call(self, assessor: SovereigntyAssessor) -> None:
        result = assessor.assess(
            data_types=[],
            regulations=[],
            org_minimum=SovereigntyLevel.L4_LOCAL_AUGMENTED,
        )
        assert result.score >= 4

    def test_instance_org_minimum_respected(self) -> None:
        assessor = SovereigntyAssessor(org_minimum=SovereigntyLevel.L5_FULLY_LOCAL)
        result = assessor.assess(data_types=[], regulations=[])
        assert result.score >= 5

    def test_gdpr_at_l6_adds_warning(self, assessor: SovereigntyAssessor) -> None:
        result = assessor.assess(data_types=[], regulations=["GDPR"], geography="EU")
        assert any("GDPR" in w for w in result.warnings)

    def test_l7_adds_airgap_warning(self) -> None:
        assessor = SovereigntyAssessor(org_minimum=SovereigntyLevel.L7_AIRGAPPED)
        result = assessor.assess(data_types=[], regulations=[])
        assert any("air-gap" in w.lower() or "L7" in w for w in result.warnings)

    def test_assessment_has_justification(self, assessor: SovereigntyAssessor) -> None:
        result = assessor.assess(data_types=["phi"], regulations=["HIPAA"])
        assert len(result.justification) > 0

    def test_assessment_data_sensitivity_field(self, assessor: SovereigntyAssessor) -> None:
        result = assessor.assess(data_types=["phi"], regulations=[])
        assert result.data_sensitivity >= 1

    def test_assessment_regulatory_drivers_populated(self, assessor: SovereigntyAssessor) -> None:
        result = assessor.assess(data_types=[], regulations=["HIPAA"])
        assert "HIPAA" in result.regulatory_drivers

    def test_assessment_capability_requirements_present(self, assessor: SovereigntyAssessor) -> None:
        result = assessor.assess(data_types=["phi"], regulations=["HIPAA"])
        assert isinstance(result.capability_requirements, dict)

    def test_max_driver_wins(self, assessor: SovereigntyAssessor) -> None:
        result_geo = assessor.assess(data_types=[], regulations=[], geography="EU")
        result_both = assessor.assess(data_types=["pii"], regulations=["GDPR"], geography="EU")
        # EU geography drives to 6; adding pii + GDPR shouldn't lower it
        assert result_both.score >= result_geo.score

    def test_score_clamped_to_1_7(self, assessor: SovereigntyAssessor) -> None:
        result = assessor.assess(data_types=[], regulations=[])
        assert 1 <= result.score <= 7

    def test_no_justification_parts_produces_baseline_text(
        self, assessor: SovereigntyAssessor
    ) -> None:
        result = assessor.assess(data_types=[], regulations=[])
        assert "baseline" in result.justification.lower() or "No sensitive" in result.justification

    def test_justification_with_drivers_mentions_driver(
        self, assessor: SovereigntyAssessor
    ) -> None:
        result = assessor.assess(data_types=["phi"], regulations=[])
        assert "data_sensitivity" in result.justification or "Data sensitivity" in result.justification

    def test_us_geography(self, assessor: SovereigntyAssessor) -> None:
        result = assessor.assess(data_types=[], regulations=[], geography="US")
        assert result.score >= 2

    def test_deployment_template_matches_score(self, assessor: SovereigntyAssessor) -> None:
        result = assessor.assess(data_types=["phi"], regulations=["HIPAA"])
        expected_template = _DEPLOYMENT_TEMPLATE_NAMES.get(result.score, "l1_cloud")
        assert result.deployment_template == expected_template


# ---------------------------------------------------------------------------
# SovereigntyAssessor.describe_level
# ---------------------------------------------------------------------------

class TestDescribeLevel:
    def test_returns_string(self, assessor: SovereigntyAssessor) -> None:
        description = assessor.describe_level(SovereigntyLevel.L3_HYBRID)
        assert isinstance(description, str)
        assert len(description) > 0

    def test_all_levels_have_description(self, assessor: SovereigntyAssessor) -> None:
        for level in SovereigntyLevel:
            desc = assessor.describe_level(level)
            assert isinstance(desc, str)



"""Unit tests for agent_sovereign.classifier.sensitivity."""
from __future__ import annotations

import pytest

from agent_sovereign.classifier.levels import SovereigntyLevel
from agent_sovereign.classifier.sensitivity import (
    DATA_SENSITIVITY,
    DataSensitivityDetector,
    DetectionResult,
)


class TestDataSensitivityMapping:
    def test_public_info_scores_one(self) -> None:
        assert DATA_SENSITIVITY["public_info"] == 1

    def test_classified_scores_seven(self) -> None:
        assert DATA_SENSITIVITY["classified"] == 7

    def test_phi_scores_five(self) -> None:
        assert DATA_SENSITIVITY["phi"] == 5

    def test_medical_records_scores_four(self) -> None:
        assert DATA_SENSITIVITY["medical_records"] == 4

    def test_financial_data_scores_three(self) -> None:
        assert DATA_SENSITIVITY["financial_data"] == 3

    def test_biometric_data_scores_five(self) -> None:
        assert DATA_SENSITIVITY["biometric_data"] == 5

    def test_itar_technical_data_scores_seven(self) -> None:
        assert DATA_SENSITIVITY["itar_technical_data"] == 7


class TestDetectionResult:
    def test_default_sovereignty_level_is_l1(self) -> None:
        result = DetectionResult()
        assert result.sovereignty_level == SovereigntyLevel.L1_CLOUD

    def test_default_max_level_is_one(self) -> None:
        result = DetectionResult()
        assert result.max_level == 1

    def test_default_detected_types_is_empty(self) -> None:
        result = DetectionResult()
        assert result.detected_types == []


class TestDataSensitivityDetectorScan:
    def setup_method(self) -> None:
        self.detector = DataSensitivityDetector()

    def test_scan_empty_string_returns_l1(self) -> None:
        result = self.detector.scan("")
        assert result.sovereignty_level == SovereigntyLevel.L1_CLOUD
        assert result.detected_types == []

    def test_scan_plain_text_returns_l1(self) -> None:
        result = self.detector.scan("The quick brown fox jumps over the lazy dog.")
        assert result.sovereignty_level == SovereigntyLevel.L1_CLOUD

    def test_scan_detects_email_address(self) -> None:
        result = self.detector.scan("Contact us at alice@example.com for support.")
        assert "customer_email" in result.detected_types

    def test_scan_email_sets_level_two(self) -> None:
        result = self.detector.scan("Email: bob@company.org")
        assert result.max_level >= 2

    def test_scan_detects_pci_card_number(self) -> None:
        result = self.detector.scan("Card number: 4111111111111111")
        assert "pci_card_data" in result.detected_types

    def test_scan_detects_cvv_keyword(self) -> None:
        result = self.detector.scan("Enter your CVV to complete the transaction.")
        assert "pci_card_data" in result.detected_types

    def test_scan_detects_medical_records(self) -> None:
        result = self.detector.scan("Patient diagnosis: ICD-10 code J18.9")
        assert "medical_records" in result.detected_types

    def test_scan_detects_patient_id(self) -> None:
        result = self.detector.scan("The patient record contains a patient ID.")
        assert "medical_records" in result.detected_types

    def test_scan_detects_phi_ssn_format(self) -> None:
        result = self.detector.scan("SSN: 123-45-6789")
        assert "phi" in result.detected_types

    def test_scan_detects_phi_keyword(self) -> None:
        result = self.detector.scan("This document contains protected health information.")
        assert "phi" in result.detected_types

    def test_scan_detects_biometric_data(self) -> None:
        result = self.detector.scan("The system uses fingerprint recognition.")
        assert "biometric_data" in result.detected_types

    def test_scan_detects_genetic_data(self) -> None:
        result = self.detector.scan("The genome sequence was analysed for SNP variants.")
        assert "genetic_data" in result.detected_types

    def test_scan_detects_classified_marker(self) -> None:
        result = self.detector.scan("TOP SECRET document regarding national security.")
        assert "classified" in result.detected_types
        assert result.sovereignty_level == SovereigntyLevel.L7_AIRGAPPED

    def test_scan_detects_itar(self) -> None:
        result = self.detector.scan("This item is ITAR controlled under USML Category XV.")
        assert "itar_technical_data" in result.detected_types

    def test_scan_detects_financial_routing_number(self) -> None:
        result = self.detector.scan("Routing number provided for wire transfer.")
        assert "financial_data" in result.detected_types

    def test_scan_detected_types_are_sorted(self) -> None:
        result = self.detector.scan(
            "Email: test@test.com. Routing number for wire transfer."
        )
        assert result.detected_types == sorted(result.detected_types)

    def test_scan_evidence_keys_match_detected_types(self) -> None:
        result = self.detector.scan("Email: test@example.com for billing.")
        for data_type in result.detected_types:
            assert data_type in result.evidence

    def test_scan_max_level_reflects_highest_type(self) -> None:
        # Classified = 7, should dominate
        result = self.detector.scan(
            "Email: a@b.com. TOP SECRET clearance required."
        )
        assert result.max_level == 7

    def test_scan_phi_dominates_over_medical_records(self) -> None:
        result = self.detector.scan(
            "Patient record with date of birth and diagnosis."
        )
        # PHI scores 5, medical_records scores 4
        assert result.max_level >= 4


class TestDataSensitivityDetectorScoreDataTypes:
    def setup_method(self) -> None:
        self.detector = DataSensitivityDetector()

    def test_empty_list_returns_one(self) -> None:
        assert self.detector.score_data_types([]) == 1

    def test_single_known_type_returns_its_score(self) -> None:
        assert self.detector.score_data_types(["medical_records"]) == 4

    def test_multiple_types_returns_max(self) -> None:
        score = self.detector.score_data_types(["employee_data", "phi"])
        assert score == 5  # phi=5 > employee_data=2

    def test_unknown_type_defaults_to_one(self) -> None:
        assert self.detector.score_data_types(["nonexistent_type"]) == 1

    def test_classified_returns_seven(self) -> None:
        assert self.detector.score_data_types(["classified"]) == 7


class TestDataSensitivityDetectorCustomisation:
    def test_custom_score_overrides_builtin(self) -> None:
        detector = DataSensitivityDetector(custom_scores={"phi": 6})
        assert detector.score_data_types(["phi"]) == 6

    def test_custom_pattern_is_detected(self) -> None:
        import re

        custom_patterns = {"financial_data": [re.compile(r"\bPROPRIETARY\b")]}
        detector = DataSensitivityDetector(custom_patterns=custom_patterns)
        result = detector.scan("This is PROPRIETARY corporate information.")
        assert "financial_data" in result.detected_types

    def test_custom_new_data_type_with_score(self) -> None:
        import re

        custom_patterns = {"trade_secret": [re.compile(r"\bTRADE_SECRET\b")]}
        custom_scores = {"trade_secret": 6}
        detector = DataSensitivityDetector(
            custom_patterns=custom_patterns,
            custom_scores=custom_scores,
        )
        result = detector.scan("Document contains TRADE_SECRET information.")
        assert "trade_secret" in result.detected_types
        assert result.max_level == 6
